#Generic methods to create documents.
#The target format ia generic, supported are LaTeX and HTML.
#
#==Elements
#A document is structured by elements.
#Each element contains attributes/parameters and content. 
#(Think of HTML-attributes and TeX-Parameters and the content of HTML-Tags and TeX-Environments.).
#
#Each element corresponds to a HTML-Tag or a LaTeX-Makro/Environment.
#There is a "translation" between HTML and LaTeX elements.
#
#Elements are defined by Element.get (or the method element() ), 
#new element types can be defined with Element.create() ).
#
#==	Examples
#See the file DocGenerator_Example.rb
#
#
#==	Open topics
#-	control content
#	(Tag br in HTML klappt).
#-	set h1 to \part, \chapter, \section...
#-	Wiki-Output
#-	ASCII-Output

#~ require 'C:/usr/Script/MyScripts' if File.exist?('C:/usr/Script/MyScripts.rb')
#~ require 'GetoptLongWithHelp'
#~ include 'DocGenerator_HTML'

#This class defines possible elements of a document.
#For each type of elements a class is defined.
#The definition can be done explicite or generic with Element.create.
#
#All types of elements are stored in a hash, 
#the elements can be created via the method Element.get (or element() ).
class Element
	#Hash with all ids and their corresponding classes.
	@@ids = Hash.new( )
	#All attributes are stored in this hash.
	#The key is the corresponding class.
	@@attr	= { Element => {} }
	#All output routines are stored in this hash.
	#The key is the corresponding class.
	@@log	= false
	def Element.log( log )
		@@log = log
	end

	#Return a element class.
	def Element.get( name )
		return @@ids[name]
	end
	#This method is called, when a subclass of this class is defined.
	#This can happen via Element.create or in another programm.
	#
	#Add subclasses to the list of all element classes and initialize some internal hashes.
	def Element.inherited( subclass )
		puts "New subclass for 'Element' created: '#{subclass}'" if @@log
		#All attributes of a class are collected in a hash.
		@@attr[subclass]	= {}
		#Fill @@ids
		#There is no possibility to give a list of id's or names to a 'inherit'-method.
		#So a special method Element.add is used.
		Element.add( [subclass], subclass )
	end
	#Add a new element class or assign another name (synonym) for a class.
	#The initial call is made by Element.inherited.
	def  Element.add( idlist, elementClass )
		if ! elementClass.new.kind_of?( Element )
			raise "Element.add get no Element-class #{elementClass}"
		end
		if ! idlist.kind_of?( Array )
			idlist = [ idlist ]
		end
		idlist.each{|id|
			if @@ids[ id ] == nil
				puts "Element class '#{id}' is new: #{elementClass}" if @@log
				#All new ids are collected
				@@ids[ id ] = elementClass
				#All ids are collected in the method Element.ids.
				if id.kind_of?( Symbol ) or id.kind_of?( String )
					elementClass.class_eval( %Q
						def ids()
							return #{( elementClass.new.ids << id ).inspect}
						end
					)
				end
			elsif	@@ids[ id ] == elementClass
				puts "Element class '#{id}' is defined double: #{elementClass}" if @@log
			else
				puts "Error: ID '#{id}' is redefined #{@@ids[id]} -> #{elementClass}"
				@@ids[id] = elementClass
			end
		}
	end
	
	#Generic creation of a new class to define a new element.
	#-	name must be defined.
	#-	attr is a hash with all attributes.
	#	For details see Element.add_attributes
	#-	content is a flag, which defines if the element contains "content".
	#-	output contains a hash with logic, how to handle the output.
	#	-:htmltag	Tag for html-output.
	#	-:latex	Template for LaTeX-output
	#	-:html		Template for HTML-output
	def  Element.create( name, attr = {}, content = false, output = {} )
		#First some checks
		if ! attr.kind_of?( Hash )
			raise "Type error Element.create: Expected Hash, get #{attr.class}"
		end
		#Generic class creation
		elementclass = Class.new( Element )
		#Add the id of the new class to central collection.
		Element.add( name, elementclass )

		#Set a flag, if the class can contain 'content'.
		elementclass.class_eval( %Q|
			def content?()
				return #{content}
			end|)
		
		output.each{ |k,v|
			case k
			when :latex
				elementclass.add_output( :latex,	v )
			when :html
				elementclass.add_output( :html,	v )
			when :text
				elementclass.add_output( :text,	v )
			when :wiki
				elementclass.add_output( :wiki,	v )
			when :htmltag
				if v
					elementclass.class_eval( "def htmltag()\n'#{v}'\nend" )
				else
					elementclass.class_eval( "def htmltag()\nnil\nend" )
				end
			else
				puts "Unknown format #{k} for #{name}"
			end
		}
		elementclass.add_attributes( attr )
		#Return the class.
		return elementclass
	end
	#Return all id's of the class.
	#This method is expanded in Element.add.
	def ids( )
		return []
	end
	#Prepares an overview on all Elements.
	#Can be used for some debugging.
	def Element.overview( list = @@ids.values.uniq )
		if ! list.kind_of?(Array)
			list = [ list ]
		end
		result = '=' * 10 + "\nElement overview:"
		list.each{ |k|
			result += "\n#{k}\n"
			result += "\tId's:\t\t"
			result += k.new.ids.join("\n\t\t\t")
			result += "\n\tAttributes:\t"
			result += "can contain "
			result += "no " if ! k.new.content?
			result += "content\n\t\t\t"
			@@attr[k].each{ |k2,v|
				result += "#{k2}:\t#{v.inspect}\n\t\t\t"
			}
		}
		return result
	end
	#Add possible attributes to an element class.
	#Attributes are defined in a hash, each key get a array with values.
	#The values can be:
	#-	OBLIGATORY/OPTIONAL
	#-	Alias (this allows the usage of a position and a name for a parameter).
	#-	allowed elements ( e.g. list content only list items)
	def Element.add_attributes( attr )
		elattr	= @@attr[self]
		attr.each{ |k,a |
			elattr[k] = a
		}
	end
	#Add an output routine.
	def Element.add_output( target, string )
		case target
		when :latex
			cmd = "def to_latex()\n"
		when :html
			cmd = "def to_html()\n"
		when :text
			cmd = "def to_text()\n"
		when :wiki
			cmd = "def to_wiki()\n"
		else
			puts "#{self}: Undefined target format #{target}"
		end
		if ! string.kind_of?( String )
			puts self.new.ids
		end
		template = string.gsub(/\\/, '\\\\\\')
		template.gsub!(/"/, '\"')
		cmd += "\"#{template}\"\n"
		cmd += "end\n"
		class_eval( cmd )
	end
	#Defines an element.
	#
	#There are two ways to add values:
	#-	"named" values in an hash.
	#	This named values can be attributes of a HTML-Tag or Parameters of a TeX-Makro.
	#	There is a check, if the attribute exist.
	#-	Content. Values "inside" the element.
	#	The content can be the content of a HTML-Tag or the content of a TeX-Environment.
	#	There is a check, if this class allows content.
	def initialize( attr={}, content = nil)
		#@attr is a hash containing the values for each attribute.
		@attr = Hash.new(  )
		@content = []	if self.content?()
		@crbefore	= false
		@crafter	= false
		#Initialize the attribute hash.
		@@attr[self.class].each{ |k,a|
			#type check on Attribute does not work .
			if a.kind_of?( Symbol )
				#Filled in the second run
				#~ @attr[k] = @attr[a]
			elsif a	#.kind_of?( Attribute )
				@attr[k] = a.new( k, self )
			else
				@attr[k] = Attribute.create().new( k, self )
			end
		}
		#Assign Alias-Attributes
		@@attr[self.class].each{ |k,a|
			next if !a.kind_of?( Symbol )
			if @attr[a]
				@attr[k] = @attr[a]
			else
				puts "Undefined Alias-Attribute #{k}"
				@attr[k] = Attribute.create().new( k, self )
			end
		}
		attr.each{ |k,v|
			if @attr[k]
				@attr[k] << v
			else
				puts "Usage of unknown attribute '#{k}' in '#{self.ids}'"
			end
		}
		self << content if  content 
	end
	#Add something to the content.
	#Only possible on elements, which supports "content".
	def << ( content )
		if @content
			@content << content
		else
			puts "Add content to an element without this feature (#{self.ids}, #{content.inspect}"
		end		
	end
	#Add content after a target (other content)
	#	insertafter(target,*obj)
	#Requires the ability of "content".
	def insertafter(target,*obj)
		self.insert(target, 1, obj)
	end
	#Add content before a target (other content)
	#	insertbefore(target,*obj)
	#Requires the ability of "content".
	def insertbefore(target,*obj)
		self.insert(target, 0, obj)
	end
	#Insert content relative to a given target.
	#The target must exist, pos defines the relative position (0=before, 1=one after the element).
	#Method called by Element#insertbefore and Element#insertafter.
	def insert(target,pos,*obj )
		if @content
			if @content.include?( target )
				@content[@content.index(target)+pos ,0] = obj
			else
				puts "Insert content not possible. Reference object does not exist (#{target.inspect})"
			end
		else
			puts "Add content to an element without this feature"
		end		
		self
	end
	#Insert an empty line after the last entry
	def cr()
		@crbefore	= true
		self
	end
	#Insert an empty line before and after the last entry
	def CR()
		@crbefore	= true
		@crafter	= true
		self
	end
	#Return an attribute to add content.
	def [] (key)
		attr = @attr[key]
		if !attr
			puts "Request unknown attribute '#{key}', return a dummy"
			attr = Attribute.create( ).new(key, self)
		end
		return attr
	end
	#Default setting, each element class may contain contents.
	#Can be redefined by Elements.create.
	def content?()
		return true
	end
	
	@@level = 0
	#Build a String to be used for the target document.
	#Calls Element#to_latex and Element#to_html.
	def to_s( target = Document.target )
		#Some checks
		@attr.each{|k,v|
			if v.required? and v.to_s == ''
				puts "#{self.ids}: Attribut '#{k}' without required value"
			end
		}
		#Build the string.
		case target
		when :latex
			result = to_latex()
		when :html
			result = to_html()
		when :wiki
			result = to_wiki()
		when :text
			result = to_text()
		when :debug
			@@level += 1
			result = "\n<element #{self.ids.join(',')} begin>"
			@attr.each{|k,v|
				#result += "\n\t#{k}: #{v.inspect}" 
				if v != []
					result += "\n\t#{k}:" 
					v.each{|v2|
						result += v2.to_s
					}
				end
			}
			result += @content.to_s
			result += "\n<element #{self.ids.join(',')} end>"
			@@level -= 1
			result.gsub!(/^/, "\t" * @@level ) 
		else
			puts "Undefined target format '#{target}'"
		end
		result = "\n" + result 	if @crafter
		result += "\n" 		if @crbefore
		return result
	end
	#This is a dummy method, called from Element#to_s for the target format LaTeX.
	#This method must be overwritten from the element class.
	#
	#By default, the concatenation of all ids and the content is taken.
	def to_latex()
		puts "Missing output routine for LaTeX (#{self.ids.join(',')})" if @@log
		return "\\#{self.ids}{#{@content}}\n"
	end
	#Build key-val options from attributs.
	def texkeyval()
		keyval = ''
		@attr.each{|k,v|
		puts k,v.inspect
		puts v.to_s.inspect
		puts v.texkeyval?
			keyval += "#{k}={#{v}} " if v.to_s != '' and v.texkeyval?
		}
		return keyval
	end
	#Method for html-output.
	#If the method is redefined, the result is taken as a tag.
	def htmltag()
		''
	end
	#This is a dummy method, called from Element#to_s for the target format HTML.
	#The tag from Element.htmltag is taken and all attributes and the content are used.
	#
	#This method can be overwriten from the element class.
	def to_html()
		tag = htmltag()
		if ! tag 
			puts "No HTML element available (#{self.ids.join(',')})" if @@log
			return ''
		elsif tag == ''
			tag = 'span' 
			puts "Missing output routine for HTML (#{self.ids.join(',')})" if @@log
		end
		html = "<#{tag} "
		@attr.each{|k,v|
			html += "#{k} = \"#{v}\" " if v.to_s != '' and v.html?
		}
		if @content
			html += ">\n#{@content}\n"
			html += "</#{tag}>\n"
		else
			html += '/>'
		end
		return html
		return self.to_s( :debug )
	end	#to_html
	def to_wiki()
		puts "Missing output routine for Wiki (#{self.ids.join(',')})" if @@log
		return "#{@content}\n"
	end
	def to_text()
		puts "Missing output routine for Text (#{self.ids.join(',')})" if @@log
		return "#{@content}"
	end
	
end

#Returns an element of a type "name".
#Attributes and content are optional parameter.
#
def element( name, attr = {}, content = nil )
	element = Element.get( name )
	if element
		element = element.new(attr, content ) if element
	else
		element = Element.create( name, {}, true  ).new(attr, content )
		puts "Usage of undefined element #{name}"
	end
	return element
end

#Administrate attributes for elements.
class Attribute
	@@settings	= {}
	@@values		= {}
	#Generic creation of a new class to define a new Atttribute.
	#Settings is a hash with generell settings.
	#Supported parameters:
	#-	:required
	#-	:content	adding values to this attribute add it to the content.
	#-	:values	allowed values
	#-	:html		used for html-output
	#-	:latex		used for latex-output
	def  Attribute.create( settings = [], values = nil )
		#First some checks
		if ! settings.kind_of?( Array )
			raise "Type error Attribute.create: Expected Array, get #{attr.class}"
		end
		#Generic class creation
		attributeclass = Class.new( Attribute )
		#Set the flags.
		settings.each{|setting|
			case setting
			when :required
				attributeclass.class_eval('def required?(); true; end')
			when :content
				attributeclass.class_eval('def content?();  true; end')
			when :unique
				attributeclass.class_eval('def unique?();  true; end')				
			when :html
				attributeclass.class_eval('def html?();  true; end')
			when :latex
				attributeclass.class_eval('def latex?();  true; end')
			when :texkeyval
				attributeclass.class_eval('def texkeyval?();  true; end')
			when :text
				attributeclass.class_eval('def text?();  true; end')
			when :wiki
				attributeclass.class_eval('def wiki?();  true; end')
			else
				puts "Attribute: Unknown setting #{setting}"
			end
		}
		if values
			attributeclass.class_eval("def allowed_values();  return #{values.inspect}; end")
		end	
		return attributeclass
	end
	
	#Each attribute is assigned to an element.
	def initialize( name, element )
		@name	= name
		@element	= element		
		@attr_content	= []
	end
	#Check if the attribute is required.
	#Redefined by Attribute.create and used in Element.to_s.
	def required?();  false; end
	#Check if the attribute can contain a "content".
	#Redefined by Attribute.create.
	def content?();  false; end
	#Values may be added only once. 
	def unique?();  false; end
	#Sometimes the content of an attribute is needed directly.
	def content();  @attr_content; end
	#Check if content was added
	def filled?();  return @attr_content.size > 0; end
	#Attribut is used for HTML.
	def html?();  false; end
	#Attribut is used for Text.
	def text?();  false; end
	#Attribut is used for Wiki.
	def wiki?();  false; end
	#Attribute is used for latex.
	#Only for informational reasons.
	def latex?();  false; end
	#Attribute is used as a keyval-option
	def texkeyval?();  false; end
	#Return all allowed values.
	#Redefined by Attribute.create and used in Attributes.<<
	#By default everything is allowed.	
	#
	#This method must return an Array with the allowed values or classes.
	def allowed_values()
		return nil
	end
	#Check if the given value is allowed.
	def allowed?( value )
		return true if allowed_values() == nil
		allowed_values().each{|v|
			if value == v
				return true
			elsif v.class == Class and value.is_a?( v )
				return true
			end
			#Check next allowed value.
		}
		#No allowed value is set.
		return false
		#~ return allowed_values().include?( value )
	end
	#Add a value to the attribute content.
	def <<( value )
		if ! self.allowed?( value )
			puts "Attribute #{@name}: Illegal value #{value} in #{@element.inspect}"
		end
		if unique? and @attr_content.size > 0
			puts "Attribute #{@name}: More then one value added (#{value}) in #{@element.inspect}"
			return self
		end
		if content?
			@element << value
		else
			@attr_content << value
		end
	end
	#Return the content.
	def to_s()
		@attr_content.to_s()
	end
	def inspect()
		return "<#Attribute #{@name}>"
	end
end

#
#Container for a document.
#
#Each document contains a header and a body. Both are special Element classes.
#
class Document
	@@templates	= {
		:include	=> %q|<<prefix>>
			% ----------------------------------------------------------------
			<<head>>
			% ----------------------------------------------------------------
			<<body>>
			% ----------------------------------------------------------------
		|.gsub(/^\t*/, ''),
		:article	=> %q|<<prefix>>
			\documentclass[<<classoptions>>]{scrartcl}
			\usepackage{babel}
			\usepackage[ansinew]{inputenc}
			\usepackage{hyperref}
			% ----------------------------------------------------------------
			<<head>>
			% ----------------------------------------------------------------
			\begin{document}
			<<body>>
			\end{document}
			% ----------------------------------------------------------------
		|.gsub(/^\t*/, ''),
		:festodoc	=> %q|<<prefix>>
			\documentclass[<<classoptions>>,lke]{festodoc}
			%\usepackage{changes}
			% ----------------------------------------------------------------
			<<head>>
			% ----------------------------------------------------------------
			\begin{document}
			<<body>>
			\end{document}
			% ----------------------------------------------------------------
		|.gsub(/^\t*/, ''),
		:report	=> '',
		:book	=> '',
		:html	=> %q|<!--
			<<prefix>>
			-->
			<!doctype html public "-//W3C//DTD HTML 4.01 Transitional//EN">
			<html>
			<<head>>
			<<body>>
			</html>
			|.gsub(/^\t*/, ''),
	}
	#Create a new document.
	#There are different document templates supported. 
	#The templates refer to the corresponding TeX-classes.
	#-	article
	#-	report
	#-	book
	def initialize( template = :article )
		self.set_template( template )
		@body	= element( :body )
		@head	= element( :head )
		@texfilename = nil
		@language	= 'ngerman'
		@options	= []
	end
	def set_template( template )
		if template.kind_of?(String )
			puts "Set Template with string"
			@template = template
			return
		end
		@template = @@templates[template]
		if ! @template
			puts "Unknown template #{template}, valid:\n\t#{@@templates.keys.join("\n\t")}"	#fixme raise
		elsif ! @template.kind_of?(String )
			puts "Template no string #{@template.inspect}"
			@template = @template.to_s
		end
	end
	#body and head are lists, containing elements.
	attr_reader	:body, :head
	#Set document description
	attr_writer	:title, :author, :date, :keywords, :description
	attr_writer	:language
	#Flag, if tex should be started immediate after document creation.
	attr_writer	:runtex

	#Constant to detect the generated header
	PREFIX_ENDFLAG = 'Generation-Info-End'
	#Global method to return the actual target type.
	#Needed by Element#to_s to decide which method is used to prepare the output.
	#
	#The usage of this technic makes similar processing unposibble.
	def Document.target()
		@@target
	end
	#Prepare the docinfo.
	#May be called only once.
	def docinfo()
		#Build docinfo.
		result = []
		result << element( :title, {}, @title) if @title
		result << element( :author, {}, @author) if @author
		result << element( :date, {}, @date) if @date
		result << element( :keywords, {}, @keywords) if @keywords
		result << element( :metadescription, {}, @description) if @description
		#If this method is called twice (e.g. with save to tex and html),
		#then the title is doubled in the second output.
		#So we redefine the method, so the second call returns nil.
		self.class.class_eval( <<-classeval
			def docinfo()
				return nil
			end
		classeval
		)
		return result
	end
	def add_option( option)
		@options << option
	end
	#Save the file.
	#The type of the document is determined by the file extensison.
	#Supported document types are:
	#-	tex
	#-	html
	#Depending on a template, different 
	def save( filename, overwrite=false, pattern = nil, replace = nil  )
	
		if ! @template
			puts "No template available"
			return false
		end
		
		old = nil
		prefix = "\n\tBuild by #{__FILE__}\n\t#{Time.now.strftime('%Y/%m/%d %H:%M:%S')}\n#{PREFIX_ENDFLAG}"
		if File.exist?( filename )
			f = File.new( filename )
			old = f.readlines().to_s
			f.close
		end
		#Determine the target document type, depending on extension.
		extension = File.basename( filename ).split( /\./).last
		case extension
		when /tex/i
			new = @template.dup
			add_option( @language )
			prefix.gsub!(/^/, '%%')
			#Delete prefix (with generation time) for later compare.
			old.sub!(/\A.*#{PREFIX_ENDFLAG}/m, '<<prefix>>' ) if old
			@@target = :latex
			@texfilename = filename
		when /htm[l]?/i
			new = @@templates[:html].dup
			#Delete prefix (with generation time) for later compare.
			old.sub!(/\A.*#{PREFIX_ENDFLAG}/m, "<!--\n<<prefix>>" ) if old
			@@target = :html
		when /txt/
			@@target = :text
			new = '<<body>>'
		#~ when /wiki/
			#~ @@target = :wiki
		else
			raise "Unknown Extension #{extension}"
		end
		if ! new.kind_of?( String )
			puts "New is wrong type: #{new.inspect}"
		end

		@head	<< self.docinfo()
		new.sub!( '<<head>>',	@head.to_s)
		new.sub!( '<<body>>',	@body.to_s)
		new.sub!( '<<classoptions>>',	@options.join(','))

		new.gsub!( pattern, replace ) if replace 

		if new != old
			new.sub!( '<<prefix>>',	prefix)
			if ( File.exist?( filename ) and !overwrite )
				puts "Datei #{filename} exist already \nContinue [yn]?"
				answer = $stdin.gets()	if $stdin.tty? #nur bei call aus DOS-Box
				if ! ( answer =~ /[YyjJ].*/ )
					puts "Bye"
					return false
				end
			end	
			f = File.new( filename, 'w' )
			f << new
			f.close
			puts "Save changed\t#{filename}"
			#Save Kopie of old version (attention, bak makes no control on tex or html)
			#~ f = File.new( filename.sub( extension, 'bak'), 'w' )
			#~ f << old
			#~ f.close
			Document.runtex( filename ) if @runtex
			return true
		elsif old
			puts "Unchanged\t#{filename}"
			return false
		end		
	end
	#Call TeX and translate the file.
	#Experimental
	#Fixme: Chain
	def Document.runtex( filename )
		begin
			require 'rtex'
		rescue LoadError
			puts "Sorry, didn't find the experimental tex translation tool to translate #{filename}"
			return false
		end
		#~ puts "Unknown texfile #{filename}" if ! filename
		tex = Chain.new()
		#~ Logger.level=3
		tex.file = filename
		tex.execute()
	end
	def Document.texify( input )
		out = input
		out.gsub!( /_/, '\_')
		return out
	end
end

#~ require 'DocGenerator_HTML' if __FILE__ != 'DocGenerator_HTML' #and __FILE__ != $0
#~ require 'DocGenerator_LaTeX' if __FILE__ != 'DocGenerator_LaTeX' #and __FILE__ != $0
require 'DocGenerator_Elements.rb'

__END__
class Test
	def required?(); true; end
end
test = Test.new()
puts test.required?
Test.class_eval('def required?(); false; end')
puts test.required?

testclass2 = Class.new( Test )
test2 = testclass2.new()
puts test2.required?
testclass2.class_eval('def required?(); true; end')
puts test2.required?
puts test.required?
