require 'DocGenerator'
require 'ferien'

STYPATH = ''

Element.create( :monat, {}, true )

#
#Kleine Struct mit Jahr und Monat.
#
class Month < Struct.new( 'Month', :year, :month )
	#Zhlt die Monate hoch.
	def to( to )
		y = self.year
		m = self.month
		while y < to.year or ( y <= to.year and m <= to.month )
				yield Month.new( y, m )
				m += 1
				if m > 12
					y += 1
					m = 1
				end
		end
	end
end

#
#Ein "Textelement" das einen Tag ausgibt. 
#Ergibt je Wochentag ein eigenes Makro, Ferientage bekommen ein Ferien angehngt.
#
class Kalendertag < Element
	Element.add( [:kalendertag],  Kalendertag)
	add_attributes( { :tag => Attribute.create( [ :html, :required ] ) } ) 
	
	Tag = Struct.new('Tag', :t1, :t2)
	@@tagliste	= {}
	#Container fr spezielle Texte.

	#Erzeugt 
	#	\montag		{1}{tag}{Wochentag}{Woche}{Feiertagbeschreibung}
	#	\montagFerien
	def to_latex()
		tag = @attr[:tag].content[0]
		
		makro = "\\"
		makro += tag.strftime_d( '%A' ).downcase
		
		makro += 'Ferien' if tag.schoolholiday?( Kalender.region )
		
		makro	+= "{#{tag.day}}"
		makro	+= tag.strftime_d( '{%a}{%A}{%W}' )
		makro	+= "{#{tag.holiday( Kalender.region )}}"
		texte	= @@tagliste[ tag.strftime_d('%Y-%m-%d') ]
		if texte
			makro	+= "{#{texte.t1}}{#{texte.t2}}"
		else
			makro	+= "{}{}"
		end
		return makro
	end
	#
	#Kalender nacht HTML.
	#Fixme: gleiche Breiten.
	#
	def to_html()
		tag = @attr[:tag].content[0]
		html	= element(:font, { :size => 1 } )
		html2	= html
		#Bug: Immer strong
		#~ if tag.schoolholiday?( Kalender.region )
			#~ html << html2 = element(:textbf)
		#~ end
		html2 <<	tag.day.to_s
		html2 <<	tag.strftime_d(" %a #{tag.holiday( Kalender.region )}")
		return html.to_s
	end
	
	#
	#Aufbau von zustzlichen Texten. Z.B.
	#-	Mllabfuhrtermine
	#-	Geburtstage
	#-	...	#
	def Kalendertag.add_text( day, t1, t2 )
		if @@tagliste[day]
			@@tagliste[day].t1 += " #{t1}"
			@@tagliste[day].t2 += " #{t2}"
		else
			@@tagliste[day] = Tag.new( t1, t2 )
		end
	end
	#Da Globale Variable, wird hier ein Lscher hingestellt.
	def Kalendertag.clear_text( )
		@@tagliste	= {}
	end
end

#
#Vorhandene Klasse aus DocGenerator erweitern um Region.
#
class Kalender < Document
	@@region	= :bawue
	@@footer	= nil
	#~ @@region = :niedersachsen
	
	#Eigentlich sollte das dem Objekt zugeordnet sein (@region).
	#Aber es wird in Kalendertag bentigt, und da habe ich keinen Zugriff auf das
	#zugehrige Dokument.
	def Kalender.region()
		@@region
	end
	def Kalender.region=( region )
		@@region = region
		puts "Ferien aus Region #{@@region} integriert"
	end
	def Kalender.footer=( footer )
		@@footer = footer
	end
	def Kalender.footer()
		@@footer
	end
	def Kalender.make( from, to, titel = nil )
		
		cal = Kalender.new( :article )
	#	cal.set_template( TEMPLATE )
		cal.add_option( 'landscape' )
		cal.add_option( 'a2paper' )
		
		cal.head << element( :usepackage, {}, STYPATH + 'schulkal' )
		#~ cal.head << element( :usepackage, { :option => 'colored' }, 'schulkal' )
		
		cal.body	<< center = element( :center )
		center	<< element( :latexdummy, {}, '~' )
		center	<< element( :vfill ).cr
		
		size = 0
		from.to(to){|m| size += 1}	
		center	<< tab = element( :tabular, { 
						:columndescription	=>  "*{#{size}}{|l}|",
						:columns			=>  size,
						:border			=> '1'
					} )
		center	<< element( :vfill )
		
		if titel
			tab << titlerow = element(:row, {:add_vspace => '1ex'}).cr
			titlerow.hline
			titlerow << element( :multicolumn, {	:columns => size, 
										:pos => 'c',
									}, [
					element( :Huge ), 
					element( :textbf, {}, titel )
					])
		end
		tab << titlerow = element(:row).cr
		titlerow.hline
		
		1.upto(31){|d|
			tab << row = element(:row).cr
			row.hline
			
			from.to(to){|m|
				row << col = element( :col ).cr
				if m == :daycount
					titlerow << element( :col ) if d == 1
					col << d
					next
				end
				if Date.exist?( m.year, m.month, d )
					day = Date.new( m.year, m.month, d )
				else
					col << nil
					next
				end
				if d == 1
					titlerow << element( :col, {}, [
								#~ element( :hfill ),
								element( :monat, {}, day.strftime_d('%B') ),
								#~ element( :hfill ),
							])
					
				end
				col << element( :kalendertag, { :tag => day})
			}
		}
		
		if Kalender.footer
			tab  << r = element( :row ).cr
			r << element( :multicolumn, 
					{	:columns	=> size, 
						#~ :colspan	=> size, 
						:pos		=> 'c',
					}, 
					Kalender.footer
				).cr
		end
		
		return cal
	end
end
